"""
Some allowed configuration parameters - not designed to be directly called by the user
"""

ALLOWED_INPUT_TYPES = [
    'text',
    'image',
    'tabular'
]

ALLOWED_HOWS = [
    'all',
    'regex',
    'keywords'
]
