"""
The aisquared.config.postprocessing subpackage contains objects for configuring how predictions are postprocessed.
"""

from .BinaryClassification import BinaryClassification
from .MulticlassClassification import MulticlassClassification
from .ObjectDetection import ObjectDetection
from .Regression import Regression
