"""
The aisquared.config.rendering subpackage contains objects for configuring how rendering of predictions is to occur.
"""

from .ImageRendering import ImageRendering
from .ObjectRendering import ObjectRendering
from .DocumentRendering import DocumentRendering
from .WordRendering import WordRendering
from .FilterRendering import FilterRendering
from .ContainerRendering import ContainerRendering
from .DoughnutChartRendering import DoughnutChartRendering
from .TableRendering import TableRendering
from .HTMLTagRendering import HTMLTagRendering
from .BarChartRendering import BarChartRendering
from .LineChartRendering import LineChartRendering
from .DashboardReplacementRendering import DashboardReplacementRendering
from .PieChartRendering import PieChartRendering
from .SOSRendering import SOSRendering
from .TextRendering import TextRendering
from .CustomRendering import CustomRendering
from .ChatRendering import ChatRendering

# from .DashboardRendering import DashboardRendering

from .utils import save_default_css
