"""
Some allowed configuration parameters - not meant to be directly called by the end user
"""

LOCATIONS = [
    None,
    'top',
    'bottom',
    'left',
    'right',
    'topright',
    'topleft',
    'bottomright',
    'bottomleft'
]

COLORS = [
    'red',
    'orange',
    'green',
    'purple',
    'yellow',
    'pink',
    'black',
    'grey',
    'white',
    'blue'
]

BADGES = [
    'circle',
    'square',
    'star',
    'underline'
]

WORD_LISTS = [
    'input',
    'result'
]

QUALIFIERS = [
    'gt',
    'lt',
    'gte',
    'lte',
    'in',
    'ne',
    'nin'
]

POSITIONS = [
    'absolute',
    'static'
]

STATIC_POSITIONS = [
    'append',
    'prepend'
]
