"""
The aisquared.config.preprocessing subpackage contains utilities to configure the preprocessing of data in the data pipeline.  It contains
 three separate subpackages, aisquared.config.preprocessing.text, aisquared.config.preprocessing.image, and aisquared.config.preprocessing.tabular, which configure the preprocessing of different types of data.
"""
