"""
The aisquared.config.analytic subpackage contains objects for packaging individual analytics.
"""

from .DeployedAnalytic import DeployedAnalytic
from .DeployedModel import DeployedModel
from .LocalModel import LocalModel
from .LocalAnalytic import LocalAnalytic
from .ReverseMLWorkflow import ReverseMLWorkflow
from .OnnxModel import OnnxModel
