"""
Additional utilities to use with the `aisquared` package. These utilities currently consist of two functions, the
`mimic_model` and `get_model` functions. They utilize functionality that exists in our open source package BeyondML
to train teacher-student models

To see in-depth examples of how to use these functions, please visit our GitHub repository at https://github.com/AISquaredInc/MimicModelExamples
"""

from .utils import mimic_model, get_model
