"""
The aisquared.config subpackage contains utilities and objects for packaging aisquared configuration steps and models.

For in-depth examples of how to build out .air files using the utilities and classes in this library, please visit our
GitHub repository at https://github.com/AISquaredInc/airFiles
"""

from .ModelConfiguration import ModelConfiguration
from .GraphConfiguration import GraphConfiguration
from .CustomObject import CustomObject
