"""
The aisquared.config.feedback subpackage contains objects for configuring feedback in aisquared models.
"""

from .SimpleFeedback import SimpleFeedback
from .BinaryFeedback import BinaryFeedback
from .MulticlassFeedback import MulticlassFeedback
from .RegressionFeedback import RegressionFeedback
from .ModelFeedback import ModelFeedback
from .QualitativeFeedback import QualitativeFeedback
